module Smooster
    class Application
      include Singleton
      include ActiveModel::Model

      attr_accessor  :config, :user_config, :site_templates_store, :media_assets_store, :pages_store

      def self.start
        self.instance
      end

      def base_dir
        Dir.pwd
      end

      def logger
        unless File.directory?("#{Dir.pwd}/.smo")
          FileUtils.mkdir_p("#{Dir.pwd}/.smo")
        end
        @logger ||= Logger.new(Dir.pwd + '/.smo/error.log', 'monthly')
      end

      def site_templates_store
        @site_templates_store ||=  YAML::Store.new("#{self.base_dir}/.smo/site_templates.yml")
      end

      def media_assets_store
        @media_assets_store ||=  YAML::Store.new("#{self.base_dir}/.smo/media_assets.yml")
      end

      def pages_store
        @pages_store ||=  YAML::Store.new("#{self.base_dir}/.smo/pages.yml")
      end

      def config
        @config ||= YAML::Store.new("#{self.base_dir}/.smo/configure.yml")
      end

      def html_folder
        config.transaction { config[:html_folder] }
      end

      def api_url
        config.transaction { config[:api_url] }
      end

      def site_id
        config.transaction { config[:site_id] }
      end

      def user_config
        @user_config ||= YAML::Store.new("#{Dir.home}/.smo/user_configure.yml")
      end

      def api_key
        user_config.transaction { user_config[:api_key] }
      end

      def update_api_key(api_key)
        user_config.transaction do
          user_config[:api_key] = api_key
        end
      end

    end
end
