module Smooster
  module Deploy
    module SiteTemplates

      def collection
        collection = []
        response.each do |file|
          body = File.read(file)
          template = SiteTemplate.new({:body => body, :path => file, :checksum => Digest::MD5.hexdigest(body)})
          template.smo_id = template.load_smo_id if template.load_smo_id.present?
          source = Nokogiri::HTML(body)
          if source.css('title').first
            if /^[a-z0-9\/\-]+$/.match(source.css('title').first)
              template.title = source.css('title').first.inner_html
              collection << template
            else
              puts "Couldn't create template, because the templates title-tag name must contain ONLY lower case letters, numbers and hyphens (-): #{source.css('title').first}".colorize(:red)
            end
          else
            puts "The file #{file} is not a valid HTMl-Document. Also be sure to add a title-tag".colorize(:red)
          end
        end
        collection
      end
      alias_method :all, :collection

      def response
        @response ||= initialize_collection
      end

      def reload!
        @response = initialize_collection
      end

      def upload_all
        self.all.each do |template|
          template.upload
        end
      end

      def method_missing(name, *args, &block)
        if collection.respond_to? name
          collection.send(name, *args)
        else
          super
        end
      end

      private
        def initialize_collection
          Dir.glob("#{Smooster::Application.instance.base_dir}/#{Smooster::Application.instance.html_folder()}/**/*.html")
        end
    end
  end
end


